require 'csv'
require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
	class Game
		attr_reader :title

		def initialize(title)
			@title = title.capitalize
			@players = []
		end
		def add_player(name)
			@players << name
		end
		def play(rounds)
			puts "There are #{@players.size} players in #{@title}:"
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points}"
			end

			1.upto(rounds) do |round|
				if block_given?
					break if yield
				end
				puts "\nRound #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end
		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end
		def high_score_entry(player)
			"#{player.name.ljust(20, '.')} #{player.score}"
		end
		def print_stats
			puts "\n#{@title} Statistics:"
			strong, wimpy = @players.partition {|player| player.strong?}
			
			puts "\n#{strong.size} strong players:"
			strong.each do |player|
				print_name_and_health(player)
			end
			
			puts "\n#{wimpy.size} wimpy players:"
			wimpy.each do |player|
				print_name_and_health(player)
			end

			@players.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\n#{total_points} total points from treasure found."

			puts "\n#{@title} High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end
		end
		def total_points
			@players.reduce(0) { |total, player| total += player.points }
		end
		def load_players(from_file)
			CSV.foreach(from_file) do |row|
				add_player(Player.new(row[0], row[1].to_i))
			end
		end
		def save_highscores(to_file="highscores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{@title} Highscores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end
	end

	if __FILE__ == $0
		game = Game.new("chess")
		puts game.title
		game.add_player(Player.new("linus"))
		game.add_player(Player.new("lucy", 50))
		game.play(2)
		game.print_stats
	end
end